/*
 * Copyright (c) 2006 Intel Corporation
 * All rights reserved.
 *
 * This file is distributed under the terms in the attached INTEL-LICENSE     
 * file. If you do not find these files, copies can be found by writing to
 * Intel Research Berkeley, 2150 Shattuck Avenue, Suite 1300, Berkeley, CA, 
 * 94704.  Attention:  Intel License Inquiry.
 */

/**
 * Class holding all data received from a mote.
 */
class Node {
    /* Data is hold in an array whose size is a multiple of INCREMENT, and
       INCREMENT itself must be a multiple of Constant.NREADINGS. This
       simplifies handling the extension and clipping of old data
       (see setEnd) */
    final static int INCREMENT = 100 * Constants.NREADINGS;
    final static int MAX_SIZE = 100 * INCREMENT; // Must be multiple of INCREMENT

    /* The mote's identifier */
    int id;

    /* Data received from the mote. data[0] is the dataStart'th sample
       Indexes 0 through dataEnd - dataStart - 1 hold data.
       Samples are 16-bit unsigned numbers, -1 indicates missing data. */
    int[] dataTemp;
    int[] dataHum;
    int dataStart, dataEnd;

    Node(int _id) {
    id = _id;
    }

    /* Update data to hold received samples newDataIndex .. newEnd.
       If we receive data with a lower index, we discard newer data
       (we assume the mote rebooted). */
    private void setEnd(int newDataIndex, int newEnd) {
    if (newDataIndex < dataStart || dataTemp == null) {
        /* New data is before the start of what we have. Just throw it
           all away and start again */
        dataStart = newDataIndex;
        dataTemp = new int[INCREMENT];
        dataHum = new int[INCREMENT];
    }
    if (newEnd > dataStart + dataTemp.length) {
        /* Try extending first */
        if (dataTemp.length < MAX_SIZE) {
        int newLength = (newEnd - dataStart + INCREMENT - 1) / INCREMENT * INCREMENT;
        if (newLength >= MAX_SIZE) {
            newLength = MAX_SIZE;
        }

        int[] newDataTemp = new int[newLength];
        int[] newDataHum = new int[newLength];
        System.arraycopy(dataTemp, 0, newDataTemp, 0, dataTemp.length);
        System.arraycopy(dataHum, 0, newDataHum, 0, dataHum.length);
        dataTemp = newDataTemp;
        dataHum = newDataHum;

        }
        if (newEnd > dataStart + dataTemp.length) {
        /* Still doesn't fit. Squish.
           We assume INCREMENT >= (newEnd - newDataIndex), and ensure
           that dataStart + data.length - INCREMENT = newDataIndex */
        int newStart = newDataIndex + INCREMENT - dataTemp.length;

        if (dataStart + dataTemp.length > newStart) {
            System.arraycopy(dataTemp, newStart - dataStart, dataTemp, 0,
                     dataTemp.length - (newStart - dataStart));
            System.arraycopy(dataHum, newStart - dataStart, dataHum, 0,
                    dataHum.length - (newStart - dataStart));
        }
        dataStart = newStart;
        }
    }
    /* Mark any missing data as invalid */
    for (int i = dataEnd < dataStart ? dataStart : dataEnd;
         i < newDataIndex; i++) {
        dataTemp[i - dataStart] = -1;
        dataHum[i - dataStart] = -1;
    }

    /* If we receive a count less than the old count, we assume the old
       data is invalid */
    dataEnd = newEnd;

    }

    /* Data received containing NREADINGS samples from messageId * NREADINGS 
       onwards */
    void update(int messageId, int[] readTemp, int[] readHum) {
    int start = messageId * Constants.NREADINGS;
    setEnd(start, start + Constants.NREADINGS);
    for (int i = 0; i < readTemp.length; i++) {
        dataTemp[start - dataStart + i] = readTemp[i];
        dataHum[start - dataStart + i] = readHum[i];
    }
    }

    /* Return value of sample x, or -1 for missing data */
    int getData(int x, boolean temp) {
    if (x < dataStart || x >= dataEnd) {
        return -1;
    } else {
    	// System.out.println("data: " + (temp ? dataTemp[x - dataStart] : dataHum[x - dataStart]));
        return temp ? dataTemp[x - dataStart] : dataHum[x - dataStart];
    }
    }

    /* Return number of last known sample */
    int maxX() {
    return dataEnd - 1;
    }
}
